package biggests.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.tree.*;

import biggests.tables.*;
import biggests.tree.*;
import biggests.utils.*;
import de.schlichtherle.io.File;
import smadeira.biclustering.*;

/**
 * <p>Title: Loading Panel</p>
 *
 * <p>Description: This panel displays options to load a
 *                 gene expression dataset, performs all the
 *                 necessary validation operations of the input and
 *                 loads the required data, adding the corresponding
 *                 nodes to the datasettree.</p>
 *
 * <p>Copyright:   Copyright (C) 2007  Joana Gonçalves
 *                 This program is free software; you can redistribute
 *                 it and/or modify it under the terms of the GNU General
 *                 Public License as published by the Free Software
 *                 Foundation; either version 3 of the License, or
 *                 any later version.
 *
 *                 This program is distributed in the hope that it will
 *                 be useful, but WITHOUT ANY WARRANTY; without even the
 *                 implied warranty of MERCHANTABILITY or FITNESS FOR A
 *                 PARTICULAR PURPOSE.  See the
 *                 <a href="http://www.gnu.org/licenses/gpl.html">
 *                 GNU General Public License</a> for more details.
 * </p>
 *
 * @author Joana P. Gonçalves
 */

public class PanelLoading extends JPanel {
    /**
     * Data delimiter: tab.
     */
    public static final int TAB = 0;

    /**
     * Data delimiter: space.
     */
    public static final int SPACE = 1;

    /**
     * Data delimiter: semicolon.
     */
    public static final int SEMICOLON = 2;

    /**
     * Type of gene expression data: cDNA.
     */
    public static final int CDNA = 0;

    /**
     * Type of gene expression data: oligonucleotides.
     */
    public static final int OLIGO = 1;

    /**
     * Default value for missings.
     */
    public static final float DEFAULT_MISSING_VALUE = Integer.MIN_VALUE;

    /**
     * The application's main frame.
     */
    private BiggestsFrame mainFrame;

    /**
     * Set of datatypes to be displayed at the datatypes combo box.
     */
    private static final String[] datatypes = {"cDNA", "Oligonucleotide"};

    /**
     * The layout of this panel.
     */
    private SpringLayout springLayout;

    /**
     * The information label of the dataset file input text field.
     */
    private JLabel datasetFileLabel;

    /**
     * Dataset file input text field.
     */
    private JTextField datasetFileTextField;

    /**
     * Browse button.
     */
    private JButton browseDatasetFileButton;

    /**
     * Organisms' list.
     */
    private JList organismList;

    /**
     * Organisms' label.
     */
    private JLabel organismLabel;

    /**
     * Datatype label.
     */
    private JLabel datatypeLabel;

    /**
     * Datatype combo box.
     */
    private JComboBox datatypeComboBox;

    /**
     * Log transformation check box.
     */
    private JCheckBox logTransformationCheckBox;

    /**
     * Index of the row which holds conditions' names (label).
     */
    private JLabel conditionsNamesRowNumberLabel;

    /**
     * Combo box with indexes of rows possibly holding conditions' names.
     */
    private JComboBox conditionsNamesRowNumberComboBox;

    /**
     * Check box to select/deselect gene ID.
     */
    private JCheckBox geneIDCheckBox;

    /**
     * Label for gene ID column index.
     */
    private JLabel geneIDColumnNumberLabel;

    /**
     * Combo box to select gene ID column index.
     */
    private JComboBox geneIDColumnNumberComboBox;

    /**
     * Label for probeset ID column index.
     */
    private JLabel probesetIDColumnNumberLabel;

    /**
     * Combo box to select probeset ID column index.
     */
    private JComboBox probesetIDColumnNumberComboBox;

    /**
     * Label for conversion file.
     */
    private JLabel conversionFileLabel;

    /**
     * Input text field for conversion file.
     */
    private JTextField conversionFileTextField;

    /**
     * Browse button for conversion file.
     */
    private JButton browseConversionFileButton;

    /**
     * Table for data preview.
     */
    private JTable previewTable;

    /**
     * Scroll for data preview table.
     */
    private JScrollPane scrollTable;

    /**
     * Expression data read from file.
     */
    private Object[][] expressionData;

    /**
     * Load button.
     */
    private JButton loadButton;

    /**
     * Creates a new loading panel.
     *
     * @param frame the main window
     */
    public PanelLoading(BiggestsFrame frame) {
        mainFrame = frame;
        try {
            jbInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Component initialization.
     *
     * @throws Exception when components cannot be initialized
     */
    private void jbInit() throws Exception {
        this.setPreferredSize(new Dimension(700, 700));
        springLayout = new SpringLayout();
        this.setLayout(springLayout);
        this.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
        Object[] defaultComboData = {"1", "2"};
        Dimension defaultComboDimension = new Dimension(50, 20);

        datasetFileLabel = new JLabel("Dataset file");
        datasetFileTextField = new JTextField(50);

        browseDatasetFileButton = new JButton("Browse...");
        browseDatasetFileButton.addActionListener(new
        PanelLoading_browseDatasetFileButton_actionAdapter(this));

        conditionsNamesRowNumberLabel = new JLabel(
                "Conditions' names in row number");
        conditionsNamesRowNumberComboBox = new JComboBox(defaultComboData);
        conditionsNamesRowNumberComboBox.setPreferredSize(defaultComboDimension);

        geneIDCheckBox = new JCheckBox("File contains gene IDs");
        geneIDCheckBox.addItemListener(
            new PanelLoading_geneIDCheckBox_itemAdapter(this));

        geneIDColumnNumberLabel = new JLabel("in column number");
        geneIDColumnNumberComboBox = new JComboBox(defaultComboData);
        geneIDColumnNumberComboBox.setPreferredSize(defaultComboDimension);

        probesetIDColumnNumberLabel = new JLabel(
                "Probeset IDs in column number");
        probesetIDColumnNumberComboBox = new JComboBox(defaultComboData);
        probesetIDColumnNumberComboBox.setPreferredSize(defaultComboDimension);

        conversionFileLabel = new JLabel("IDs conversion file");
        conversionFileTextField = new JTextField(27);
        conversionFileTextField.setPreferredSize(new Dimension(
                (int) conversionFileTextField.getPreferredSize().getWidth(),
                (int) browseDatasetFileButton.getPreferredSize().getHeight()));

        browseConversionFileButton = new JButton("Browse...");
        browseConversionFileButton.addActionListener(new
                PanelLoading_browseConversionFileButton_actionAdapter(this));
        datasetFileTextField.setPreferredSize(new Dimension(
                (int) datasetFileTextField.getPreferredSize().getWidth(),
                (int) browseDatasetFileButton.getPreferredSize().getHeight()));
        datasetFileTextField.addCaretListener(new
                                              PanelLoading_datasetFileTextField_caretAdapter(this));
        datasetFileTextField.addInputMethodListener(new
                PanelLoading_datasetFileTextField_inputMethodAdapter(this));

        datatypeLabel = new JLabel("Datatype");
        datatypeComboBox = new JComboBox(datatypes);

        logTransformationCheckBox = new JCheckBox("Log Transformation", false);

        organismLabel = new JLabel("Organism");
        organismList = new JList(PanelFunctionAnalysis.organisms);
        organismList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        organismList.setVisibleRowCount(12);
        JScrollPane scrollList = new JScrollPane(organismList);
        organismList.setSelectedValue(PanelFunctionAnalysis.organisms[38], true);

        previewTable = new JTable(0, 0);

        scrollTable = new JScrollPane(previewTable);
        scrollTable.setPreferredSize(new Dimension(650, 200));
        scrollTable.getVerticalScrollBar().
                setUnitIncrement(mainFrame.SCROLL_BAR_INCREMENT/2);

        loadButton = new JButton("Load");
        loadButton.setPreferredSize(new Dimension(650, 70));
        loadButton.addActionListener(new PanelLoading_loadButton_actionAdapter(this));

        this.setPreviewEnabled(false);
        this.add(datasetFileLabel);
        this.add(datasetFileTextField);
        springLayout.putConstraint(SpringLayout.NORTH, datasetFileTextField, 5,
                                   SpringLayout.SOUTH, datasetFileLabel);
        springLayout.putConstraint(SpringLayout.WEST, datasetFileTextField, 0,
                                   SpringLayout.WEST, datasetFileLabel);

        this.add(browseDatasetFileButton);
        springLayout.putConstraint(SpringLayout.SOUTH, browseDatasetFileButton,
                                   0,
                                   SpringLayout.SOUTH, datasetFileTextField);
        springLayout.putConstraint(SpringLayout.WEST, browseDatasetFileButton,
                                   10,
                                   SpringLayout.EAST, datasetFileTextField);

        this.add(scrollTable);
        springLayout.putConstraint(SpringLayout.NORTH, scrollTable, 30,
                                   SpringLayout.SOUTH, datasetFileTextField);
        springLayout.putConstraint(SpringLayout.WEST, scrollTable, 0,
                                   SpringLayout.WEST, datasetFileTextField);

        this.add(organismLabel);
        springLayout.putConstraint(SpringLayout.NORTH, organismLabel, 30,
                                   SpringLayout.SOUTH, scrollTable);
        springLayout.putConstraint(SpringLayout.WEST, organismLabel, 0,
                                   SpringLayout.WEST, scrollTable);

        this.add(scrollList);
        springLayout.putConstraint(SpringLayout.NORTH, scrollList, 5,
                                   SpringLayout.SOUTH, organismLabel);
        springLayout.putConstraint(SpringLayout.WEST, scrollList, 0,
                                   SpringLayout.WEST, organismLabel);

        this.add(this.conditionsNamesRowNumberLabel);
        springLayout.putConstraint(SpringLayout.NORTH,
                                   conditionsNamesRowNumberLabel, 0,
                                   SpringLayout.NORTH, scrollList);
        springLayout.putConstraint(SpringLayout.WEST,
                                   conditionsNamesRowNumberLabel, 30,
                                   SpringLayout.EAST, scrollList);

        this.add(this.conditionsNamesRowNumberComboBox);
        springLayout.putConstraint(SpringLayout.SOUTH,
                                   conditionsNamesRowNumberComboBox, 0,
                                   SpringLayout.SOUTH,
                                   conditionsNamesRowNumberLabel);
        springLayout.putConstraint(SpringLayout.WEST,
                                   conditionsNamesRowNumberComboBox, 5,
                                   SpringLayout.EAST,
                                   conditionsNamesRowNumberLabel);

        this.add(this.geneIDCheckBox);
        springLayout.putConstraint(SpringLayout.NORTH, geneIDCheckBox, 10,
                                   SpringLayout.SOUTH,
                                   conditionsNamesRowNumberLabel);
        springLayout.putConstraint(SpringLayout.WEST, geneIDCheckBox, 0,
                                   SpringLayout.WEST,
                                   conditionsNamesRowNumberLabel);

        this.add(this.geneIDColumnNumberLabel);
        springLayout.putConstraint(SpringLayout.SOUTH, geneIDColumnNumberLabel,
                                   -4,
                                   SpringLayout.SOUTH, geneIDCheckBox);
        springLayout.putConstraint(SpringLayout.WEST, geneIDColumnNumberLabel,
                                   5,
                                   SpringLayout.EAST, geneIDCheckBox);

        this.add(this.geneIDColumnNumberComboBox);
        springLayout.putConstraint(SpringLayout.SOUTH,
                                   geneIDColumnNumberComboBox, 0,
                                   SpringLayout.SOUTH, geneIDColumnNumberLabel);
        springLayout.putConstraint(SpringLayout.WEST,
                                   geneIDColumnNumberComboBox, 5,
                                   SpringLayout.EAST, geneIDColumnNumberLabel);

        this.add(this.probesetIDColumnNumberLabel);
        springLayout.putConstraint(SpringLayout.NORTH,
                                   probesetIDColumnNumberLabel, 10,
                                   SpringLayout.SOUTH, geneIDCheckBox);
        springLayout.putConstraint(SpringLayout.WEST,
                                   probesetIDColumnNumberLabel, 0,
                                   SpringLayout.WEST, geneIDCheckBox);

        this.add(this.probesetIDColumnNumberComboBox);
        springLayout.putConstraint(SpringLayout.SOUTH,
                                   probesetIDColumnNumberComboBox, 0,
                                   SpringLayout.SOUTH,
                                   probesetIDColumnNumberLabel);
        springLayout.putConstraint(SpringLayout.WEST,
                                   probesetIDColumnNumberComboBox, 5,
                                   SpringLayout.EAST,
                                   probesetIDColumnNumberLabel);

        this.add(this.conversionFileLabel);
        springLayout.putConstraint(SpringLayout.NORTH, conversionFileLabel, 10,
                                   SpringLayout.SOUTH,
                                   probesetIDColumnNumberLabel);
        springLayout.putConstraint(SpringLayout.WEST, conversionFileLabel, 0,
                                   SpringLayout.WEST,
                                   probesetIDColumnNumberLabel);

        this.add(this.conversionFileTextField);
        springLayout.putConstraint(SpringLayout.NORTH, conversionFileTextField,
                                   5,
                                   SpringLayout.SOUTH, conversionFileLabel);
        springLayout.putConstraint(SpringLayout.WEST, conversionFileTextField,
                                   30,
                                   SpringLayout.EAST, scrollList);

        this.add(this.browseConversionFileButton);
        springLayout.putConstraint(SpringLayout.SOUTH,
                                   browseConversionFileButton, 0,
                                   SpringLayout.SOUTH, conversionFileTextField);
        springLayout.putConstraint(SpringLayout.WEST,
                                   browseConversionFileButton, 10,
                                   SpringLayout.EAST, conversionFileTextField);

        this.add(datatypeLabel);
        springLayout.putConstraint(SpringLayout.NORTH, datatypeLabel, 30,
                                   SpringLayout.SOUTH, conversionFileTextField);
        springLayout.putConstraint(SpringLayout.WEST, datatypeLabel, 30,
                                   SpringLayout.EAST, scrollList);

        this.add(datatypeComboBox);
        springLayout.putConstraint(SpringLayout.NORTH, datatypeComboBox, 5,
                                   SpringLayout.SOUTH, datatypeLabel);
        springLayout.putConstraint(SpringLayout.WEST, datatypeComboBox, 0,
                                   SpringLayout.WEST, datatypeLabel);

        this.add(logTransformationCheckBox);
        springLayout.putConstraint(SpringLayout.SOUTH,
                                   logTransformationCheckBox, 0,
                                   SpringLayout.SOUTH, datatypeComboBox);
        springLayout.putConstraint(SpringLayout.WEST, logTransformationCheckBox,
                                   10,
                                   SpringLayout.EAST, datatypeComboBox);

        this.add(loadButton);
        springLayout.putConstraint(SpringLayout.NORTH, loadButton, 20,
                                   SpringLayout.SOUTH, scrollList);
        springLayout.putConstraint(SpringLayout.WEST, loadButton, 0,
                                   SpringLayout.WEST, scrollList);
    }

    /**
     * Clears this loading functionality panel (sets the loading panel ready
     * to load a new dataset file).
     * Clears all modifications that the user may have done in text fields,
     * combo boxes and check boxes.
     */
    public void clear() {
        datasetFileTextField.setText("");
        organismList.setSelectedValue(PanelFunctionAnalysis.organisms[38], true);
        datatypeComboBox.setSelectedIndex(0);
        expressionData = null;
        logTransformationCheckBox.setSelected(false);
        previewTable = new JTable(0, 0);
        scrollTable.setViewportView(previewTable);
        this.setGeneIDCheckBoxSelected(true);
        this.setPreviewEnabled(false);
    }

    /**
     * Sets preview option enabled/disabled.
     *
     * @param enabled <code>boolean true</code> if preview option
     * should be enabled, <code>false</code> otherwise
     */
    public void setPreviewEnabled(boolean enabled) {
        organismList.setEnabled(enabled);
        organismLabel.setEnabled(enabled);
        datatypeLabel.setEnabled(enabled);
        datatypeComboBox.setEnabled(enabled);
        logTransformationCheckBox.setEnabled(enabled);

        conditionsNamesRowNumberLabel.setEnabled(enabled);
        conditionsNamesRowNumberComboBox.setEnabled(enabled);
        geneIDCheckBox.setEnabled(enabled);
        geneIDCheckBox.setSelected(true);
        geneIDColumnNumberLabel.setEnabled(enabled);
        geneIDColumnNumberComboBox.setEnabled(enabled);
        probesetIDColumnNumberLabel.setEnabled(false);
        probesetIDColumnNumberComboBox.setEnabled(false);
        conversionFileLabel.setEnabled(false);
        conversionFileTextField.setEnabled(false);
        browseConversionFileButton.setEnabled(false);
        loadButton.setEnabled(enabled);
    }

    /**
     * Sets gene ID check box selected/deselected.
     *
     * @param selected <code>boolean true</code> if gene ID
     * check box should be selected, <code>false</code> otherwise
     */
    public void setGeneIDCheckBoxSelected(boolean selected) {
        geneIDCheckBox.setSelected(selected);

        geneIDColumnNumberLabel.setEnabled(selected);
        geneIDColumnNumberComboBox.setEnabled(selected);
        probesetIDColumnNumberLabel.setEnabled(!selected);
        probesetIDColumnNumberComboBox.setEnabled(!selected);
        conversionFileLabel.setEnabled(!selected);
        conversionFileTextField.setEnabled(!selected);
        browseConversionFileButton.setEnabled(!selected);
    }

    /**
     * Finds the columns that contain names (possibly holding conditions' names).
     *
     * @param rowsWithNames <code>ArrayList</code> the set of indexes
     *        which correspond to rows with names
     * @return ArrayList the set of indexes which correspond to columns with names
     * @throws Exception
     */
    private ArrayList<Integer> findColumnsWithNames(ArrayList<Integer> rowsWithNames) throws
            Exception {
        ArrayList<Integer> columnsWithNames = new ArrayList<Integer>();
        for (int index = 0;
                         index < expressionData[rowsWithNames.
                         get(rowsWithNames.size()-1).intValue() + 1].length;
                         index++) {
            Object cell = expressionData[rowsWithNames.
                          get(rowsWithNames.size()-1).intValue() +1][index];
            if (cell instanceof String) {
                columnsWithNames.add(Integer.valueOf(index));
            }
        }
        if (columnsWithNames.size() < 1) {
            String[] options = {"Assume first column", "Generate", "Cancel"};
            int option = JOptionPane.showOptionDialog(this,
                    "Unable to find any column with " +
                    "genes' names.\n" +
                    "Assume first column contains " +
                    "genes' names or generate them?",
                    "No genes' names column",
                    JOptionPane.YES_NO_CANCEL_OPTION,
                    JOptionPane.INFORMATION_MESSAGE,
                    null,
                    options,
                    options[0]);
            switch (option) {
            case 0:
                columnsWithNames.add(Integer.valueOf(0));
                for (int i = 0; i < expressionData.length; i++) {
                    expressionData[i][0] = expressionData[i][0].toString();
                }
                break;
            case 1:
                columnsWithNames.add(Integer.valueOf(0));
                Object[][] newExpressionData = new Object
                                               [expressionData.length]
                                               [expressionData[0].length + 1];
                int genes = 0;
                for (int i = 0; i < newExpressionData[0].length; i++) {
                    if (expressionData[i][0] instanceof String) {

                    } else {
                        genes++;
                        newExpressionData[i][0] = "Gene " + genes;
                    }
                }
                for (int i = 0; i < newExpressionData.length; i++) {
                    for (int j = 1; j < newExpressionData[0].length; j++) {
                        newExpressionData[i][j] = expressionData[i][j - 1];
                    }
                }
                expressionData = newExpressionData;
                break;
            default:
                throw new Exception("");
            }
        }
        if (columnsWithNames.get(columnsWithNames.size()-1).intValue() + 1 >=
            expressionData[0].length) {
            throw new Exception("There are no columns with expression values " +
                                "after genes' names columns.");
        }

        return columnsWithNames;
    }

    /**
     * Finds the rows that contain names (possibly holding genes' names).
     *
     * @return ArrayList the set of indexes which correspond to rows with names
     * @throws Exception
     */
    private ArrayList<Integer> findRowsWithNames() throws Exception {
        ArrayList<Integer> rowsWithNames = new ArrayList<Integer>();
        int numberOfColumnsWithNames = 0;

        for (int i = 0; i < expressionData.length; i++) {
            numberOfColumnsWithNames = 0;
            Object[] row = expressionData[i];
            for (int j = 0; j < row.length; j++) {
                if (row[j] instanceof String) {
                    numberOfColumnsWithNames++;
                }
            }
            if (numberOfColumnsWithNames == expressionData[0].length) {
                rowsWithNames.add(Integer.valueOf(i));
            }
        }

        if (rowsWithNames.size() < 1) {
            String[] options = {"Assume first row", "Generate", "Cancel"};
            int option = JOptionPane.showOptionDialog(this,
                    "Unable to find any row with " +
                    "conditions' names.\n" +
                    "Assume first row contains " +
                    "conditions' names or generate them?",
                    "No conditions' names row",
                    JOptionPane.YES_NO_CANCEL_OPTION,
                    JOptionPane.INFORMATION_MESSAGE,
                    null,
                    options,
                    options[0]);
            switch (option) {
            case 0:
                rowsWithNames.add(Integer.valueOf(0));
                for (int j = 0; j < expressionData[0].length; j++) {
                    expressionData[0][j] = expressionData[0][j].toString();
                }
                break;
            case 1:
                rowsWithNames.add(Integer.valueOf(0));
                Object[][] newExpressionData = new Object
                                               [expressionData.length + 1]
                                               [expressionData[0].length];
                int conditions = 0;
                for (int j = 0; j < newExpressionData[0].length; j++) {
                    if (expressionData[0][j] instanceof String) {
                        newExpressionData[0][j] = "Genes " + j;
                    } else {
                        conditions++;
                        newExpressionData[0][j] = "C " + conditions;
                    }
                }
                for (int i = 1; i < newExpressionData.length; i++) {
                    for (int j = 0; j < newExpressionData[0].length; j++) {
                        newExpressionData[i][j] = expressionData[i - 1][j];
                    }
                }
                expressionData = newExpressionData;
                break;
            default:
                throw new Exception("");
            }
        }
        if (rowsWithNames.get(rowsWithNames.size()-1).intValue() + 1 >= expressionData.length) {
            throw new Exception("There are no rows with expression values " +
                                "after conditions' names rows.");
        }

        return rowsWithNames;
    }

    /**
     * Reads expression data from a text file.
     *
     * @param filePath <code>String</code> the path to the file with
     *        gene expression data
     * @param parameterName <code>String</code> the name/type of the file to read
     * @param delimiter <code>char</code> the delimiter of the data in the file
     * @param missingValue the <code>float</code> value to mark missing values with
     * @return Object[][] the gene expression data matrix
     * @throws FileNotFoundException if the file to read is not found
     * @throws IOException if file can't be read
     * @throws Exception if another problem ocurred while reading the file
     *         or initializing the gene expression data matrix
     */
    private Object[][] readExpressionDataFromFileTXT(String filePath,
            String parameterName,
            char delimiter,
            float missingValue) throws FileNotFoundException, IOException,
            Exception {
        if (filePath == null || filePath.equals("")) {
            throw new Exception("Please indicate a " + parameterName + ".");
        }

        File file = new File(filePath);
        if (!file.isFile()) {
            throw new Exception("Please indicate a valid " + parameterName +
                                ".");
        }

        try {
            expressionData = BiggestsIO.readExpressionDataFromFileTXT(
                    filePath, delimiter, missingValue);
        } catch (IOException ex) {
            throw new Exception("Unable to read " + parameterName + ".");
        }
        return expressionData;
    }

    /**
     * Extracts and returns genes' names from the gene expression data
     * matrix previously read.
     *
     * @param columnWithGenesNames <code>int</code> the index of the
     *        column with genes names
     * @param rowsWithNames <code>ArrayList</code> the set of indexes of
     *        the rows which contain names (rows with conditions names)
     * @return String[] the set of genes names
     */
    private String[] constructGenesNames(int columnWithGenesNames,
                                         ArrayList<Integer> rowsWithNames) {
        int startRow = rowsWithNames.get(rowsWithNames.size()-1).intValue() + 1;
        String[] genesNames = new String[expressionData.length - startRow];
        int g = 0;
        int missing = 0;
        for (int i = startRow;
                     i < expressionData.length; i++) {
            try {
                genesNames[g] = (String) expressionData[i][columnWithGenesNames];
            } catch (ClassCastException ex) {
                genesNames[g] = "";
                missing++;
            }
            g++;
        }
        if (missing > 0) {
            JOptionPane.showMessageDialog(this,
                                          missing +
                                          " genes' names were missing.",
                                          "Some genes' names missing.",
                                          JOptionPane.INFORMATION_MESSAGE);
        }
        return genesNames;
    }

    /**
     * Extracts and returns conditions' names from the gene expression data
     * matrix previously read.
     *
     * @param rowWithConditionsNames <code>int</code> the index of the
     *        row with conditions names
     * @param columnsWithNames <code>ArrayList<Integer></code> the set of indexes of
     *        the columns which contain names (columns with genes names)
     * @return String[] the set of conditions names
     */
    private String[] constructConditionsNames(
            int rowWithConditionsNames,
            ArrayList<Integer> columnsWithNames) {
        int startColumn = columnsWithNames.get(columnsWithNames.size()-1).intValue() + 1;
        String[] conditionsNames = new String[expressionData[0].length -
                                   startColumn];
        int c = 0;
        for (int i = startColumn; i < expressionData[0].length; i++) {
            conditionsNames[c] = (String) expressionData[rowWithConditionsNames][
                                 i];
            c++;
        }
        return conditionsNames;
    }

    /**
     * Extracts and returns the gene expression matrix (only expression
     * values, no genes or conditions names) from the gene expression
     * data previously read.
     *
     * @param rowsWithNames <code>ArrayList<Integer></code> the indexes
     *        of the rows with names (conditions names)
     * @param columnsWithNames <code>ArrayList<Integer></code> the
     *        indexes of the columns with names (genes names)
     * @return float[][] the gene expression matrix
     */
    private float[][] constructGeneExpressionMatrix(
            ArrayList<Integer> rowsWithNames, ArrayList<Integer> columnsWithNames) {
        int startRow = rowsWithNames.get(rowsWithNames.size()-1).intValue() + 1;
        int startColumn = columnsWithNames.get(columnsWithNames.size()-1).intValue() + 1;
        float[][] geneExpressionMatrix = new
                                         float[expressionData.length - startRow]
                                         [expressionData[0].length -
                                         startColumn];
        int g = 0;
        int c;
        for (int i = startRow; i < expressionData.length; i++) {
            c = 0;
            for (int j = startColumn; j < expressionData[0].length; j++) {
                geneExpressionMatrix[g][c] = ((Float) expressionData[i][j]).
                                             floatValue();
                c++;
            }
            g++;
        }
        return geneExpressionMatrix;
    }

    /**
     * Converts probeset IDs to gene IDs according to a conversion file.
     * Basically the probeset IDs in column <code>probesetIDColumn</code>
     * of the gene expression data are replaced by their corresponding
     * gene IDs, according to conversion file. Conversion file is a
     * text file in which data is delimited by a <code>separator</code>
     * character and is structured like this:
     * ProbesetID1      GeneID1
     * ProbesetID2      GeneID2
     * ProbesetID3      GeneID3
     * ProbesetID4      GeneID4
     *
     * @param filePath <code>String</code> the conversion file path
     * @param separator <code>char</code> the delimiter of the data
     * @param probesetIDColumn <code>int</code> the index of the
     *        column of the gene expression data that contains the
     *        probeset IDs to be converted
     * @param rowsWithNames <code>ArrayList<Integer></code> the indexes
     *        of the gene expression data rows with names
     * @throws FileNotFoundException if conversion file can't be found
     * @throws IOException if conversion file can't be read
     * @throws Exception if another problem occurs
     */
    private void convertProbesetIDToGeneID(
            String filePath,
            char separator,
            int probesetIDColumn,
            ArrayList<Integer> rowsWithNames) throws FileNotFoundException,
            IOException, Exception {
        if (filePath == null || filePath.equals("")) {
            throw new Exception("No conversion file.");
        }

        File file = new File(filePath);
        if (!file.isFile()) {
            throw new Exception("Invalid conversion file path.");
        }

        String[][] conversionEntries = BiggestsIO.
                                       readConversionEntriesFromFileTXT(
                filePath, separator);
        int converted = 0;
        for (int i = rowsWithNames.get(rowsWithNames.size()-1).intValue() + 1;
                     i < expressionData.length; i++) {
            for (int j = 0; j < conversionEntries.length; j++) {
                if (conversionEntries[j][0].equals((String) expressionData[i][
                        probesetIDColumn])) {
                    expressionData[i][probesetIDColumn] = conversionEntries[j][
                            1];
                    converted++;
                }
            }
        }
        if (converted < expressionData.length - rowsWithNames.size()) {
            throw new Exception("Some genes are missing in conversion file.");
        }
    }

    /**
     * Reads expression data from the dataset file in
     * <code>datasetFileTextField</code> and fills preview table with it.
     */
    private void preview() {
        String filePath = datasetFileTextField.getText();

        try {
            expressionData = this.readExpressionDataFromFileTXT(
                    filePath, "dataset file", '\t',
                    PanelLoading.DEFAULT_MISSING_VALUE);
        } catch (Exception ex) {
            try {
                expressionData = this.readExpressionDataFromFileTXT(
                        filePath, "dataset file", ';',
                        PanelLoading.DEFAULT_MISSING_VALUE);
            } catch (Exception excep) {
                try {
                    expressionData = this.readExpressionDataFromFileTXT(
                            filePath, "dataset file", ' ',
                            PanelLoading.DEFAULT_MISSING_VALUE);
                } catch (Exception exception) {
                    ex.printStackTrace();
                    excep.printStackTrace();
                    exception.printStackTrace();
                    JOptionPane.showMessageDialog(this,
                                                  exception.getMessage(),
                                                  "Error reading file",
                                                  JOptionPane.
                                                  INFORMATION_MESSAGE);
                    return;
                }
            }
        }
        ArrayList<Integer> columnsWithNames = new ArrayList<Integer>();
        ArrayList<Integer> rowsWithNames = new ArrayList<Integer>();

        try {
            if (expressionData.length < 2) {
                throw new Exception("It has only one row.");
            }
            if (expressionData[0].length < 2) {
                throw new Exception(
                        "It has less than 2 columns.");
            }

            rowsWithNames = this.findRowsWithNames();
            columnsWithNames = this.findColumnsWithNames(rowsWithNames);
        } catch (Exception ex) {
            ex.printStackTrace();
            JOptionPane.showMessageDialog(this,
                                          "Unable to read expression matrix.\n"
                                          + ex.getMessage(),
                                          "Error reading expression matrix",
                                          JOptionPane.INFORMATION_MESSAGE);
            return;
        }

        this.conditionsNamesRowNumberComboBox.setModel(new DefaultComboBoxModel(
                rowsWithNames.toArray()));
        this.conditionsNamesRowNumberComboBox.setSelectedIndex(rowsWithNames.
                size() - 1);
        this.geneIDColumnNumberComboBox.setModel(new DefaultComboBoxModel(
                columnsWithNames.toArray()));
        this.geneIDColumnNumberComboBox.setSelectedIndex(columnsWithNames.size() -
                1);
        this.probesetIDColumnNumberComboBox.setModel(new DefaultComboBoxModel(
                columnsWithNames.toArray()));
        this.probesetIDColumnNumberComboBox.setSelectedIndex(columnsWithNames.
                size() - 1);

        Object[] columnIndexes = new Object[expressionData[0].length];
        for (int i = 0; i < columnIndexes.length; i++) {
            columnIndexes[i] = Integer.valueOf(i);
        }

        previewTable = new TablePreview(expressionData, columnIndexes,
                                        rowsWithNames, columnsWithNames);
        scrollTable.setViewportView(previewTable);
        Enumeration<TableColumn> en = previewTable.getColumnModel().getColumns();
        int count = previewTable.getColumnModel().getColumnCount();
        int width = (int) scrollTable.getPreferredSize().getWidth() - 12;
        if (count * 80 < width) {
            while (en.hasMoreElements()) {
                en.nextElement().setPreferredWidth(width / count);
            }
        } else {
            while (en.hasMoreElements()) {
                en.nextElement().setPreferredWidth(80);
            }
        }
        this.setPreviewEnabled(true);
        mainFrame.setVisible(true);
    }

    /**
     * Tests dataset file path in <code>datasetFileTextField</code>.
     * Returns <code>true</code> if file path corresponds to a valid file,
     * <code>false</code> otherwise.
     *
     * @return <code>boolean</code> <code>true</code> if file path in
     *         <code>datasetFileTextField</code> corresponds to a valid file,
     *         <code>false</code> otherwise
     */
    public boolean testDatasetFilePath() {
        File file = new File(this.datasetFileTextField.getText());
        if (file.exists() && file.isFile()) {
            return true;
        }
        return false;
    }

//******************************************************************************
//******************************************************************************
//******************************************************************************
//       EVENT HANDLERS SECTION
//******************************************************************************
//******************************************************************************
//******************************************************************************


    /**
     * Handles the mouse event fired by a user's click on browse conversion file button.
     *
     * @param e <code>ActionEvent</code> fired by a user's click on browse conversion file button
     */
    public void browseConversionFileButton_actionPerformed(ActionEvent e) {
        if (!browseConversionFileButton.isEnabled()) {
            return;
        }

        JFileChooser conversionFileChooser = new JFileChooser("Datasets");
        try {
            conversionFileChooser.setFileFilter(new FileFilterExtension(".txt"));
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        conversionFileChooser.setAcceptAllFileFilterUsed(false);
        int option = conversionFileChooser.showOpenDialog(this);

        if (option == JFileChooser.APPROVE_OPTION) {
            conversionFileTextField.setText(conversionFileChooser.
                                            getSelectedFile().
                                            getPath());
        }
    }

    /**
     * Handles the event fired by a change in gene ID's check box selection.
     *
     * @param e the <code>ItemEvent</code> fired by a change in gene ID's check box selection
     */
    public void geneIDCheckBox_itemStateChanged(ItemEvent e) {
        this.setGeneIDCheckBoxSelected(geneIDCheckBox.isSelected());
    }

    /**
     * Handles the event fired by a change in <code>datasetFileTextField</code>'s text.
     *
     * @param event <code>InputMethodEvent</code> event fired by a change in
     *              <code>datasetFileTextField</code>'s text
     */
    public void datasetFileTextField_inputMethodTextChanged(InputMethodEvent
            event) {
        if (this.testDatasetFilePath()) {
            this.preview();
        } else {
            this.previewTable = new JTable(0, 0);
            this.scrollTable.setViewportView(previewTable);
            this.setPreviewEnabled(false);
        }
    }

    /**
     * Handles events fired by changes in caret position.
     *
     * @param event <code>InputMethodEvent</code> fired when the caret
     *        within composed text has changed
     */
    public void datasetFileTextField_caretPositionChanged(InputMethodEvent
            event) {
        if (this.testDatasetFilePath()) {
            this.preview();
        } else {
            this.previewTable = new JTable(0, 0);
            this.scrollTable.setViewportView(previewTable);
            this.setPreviewEnabled(false);
        }
    }

    /**
     * Handles an event fired when the caret position is updated.
     *
     * @param e <code>CaretEvent</code> fired when the caret position is updated
     */
    public void datasetFileTextField_caretUpdate(CaretEvent e) {
        if (this.testDatasetFilePath()) {
            this.preview();
        } else {
            this.previewTable = new JTable(0, 0);
            this.scrollTable.setViewportView(previewTable);
            this.setPreviewEnabled(false);
        }
    }

    /**
     * Handles the mouse event fired by a user's click on load button.
     *
     * @param e <code>ActionEvent</code> fired by a user's click on load button
     */
    public void loadButton_actionPerformed(ActionEvent e) {
        String organism = new String();
        String dataType = new String();
        String geneID = "Gene ID";
        char dataSeparator = '\t';
        String[] genesNames = null;
        String[] conditionsNames = null;
        float[][] geneExpressionMatrix = null;

        organism = (String) organismList.getSelectedValue();
        dataType = (String)this.datatypeComboBox.getSelectedItem();
        ArrayList<Integer> rowsWithNames = null;
        ArrayList<Integer> columnsWithNames = null;
        int conditionsNamesRow = -1;
        try {
            rowsWithNames = this.findRowsWithNames();
            columnsWithNames = this.findColumnsWithNames(rowsWithNames);

            conditionsNamesRow = ((Integer)this.
                                  conditionsNamesRowNumberComboBox.
                                  getSelectedItem()).intValue();
            if (!rowsWithNames.contains(Integer.valueOf(conditionsNamesRow))) {
                throw new Exception(
                        "The selected row for the names of the time points is incorrect.");
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            JOptionPane.showMessageDialog(this,
                                          "Did not recognize " +
                                          "expression data format.\n" +
                                          ex.getMessage(),
                                          "Unrecognized expression data format",
                                          JOptionPane.ERROR_MESSAGE);
            return;
        }

        int genesNamesColumn;
        if (this.geneIDCheckBox.isSelected()) {
            genesNamesColumn = ((Integer)this.geneIDColumnNumberComboBox.
                                getSelectedItem()).intValue();
            if (!columnsWithNames.contains(Integer.valueOf(genesNamesColumn))) {
                JOptionPane.showMessageDialog(this,
                                              "The selected column for genes' names is incorrect.",
                                              "Genes' names column is invalid",
                                              JOptionPane.ERROR_MESSAGE);
                return;
            }
        } else {
            genesNamesColumn = ((Integer)this.probesetIDColumnNumberComboBox.
                                getSelectedItem()).intValue();
            if (!columnsWithNames.contains(Integer.valueOf(genesNamesColumn))) {
                JOptionPane.showMessageDialog(this,
                                              "The selected column for genes' names is incorrect.",
                                              "Genes' names column is invalid",
                                              JOptionPane.ERROR_MESSAGE);
                return;
            }
            String conversionFilePath = this.conversionFileTextField.getText();
            geneID = "Probeset ID";
            try {
                this.convertProbesetIDToGeneID(
                        conversionFilePath,
                        dataSeparator,
                        genesNamesColumn,
                        rowsWithNames);
                geneID = "Gene ID";
            } catch (Exception ex) {
                int option = JOptionPane.showOptionDialog(
                        this,
                        "Unable to convert genes' names. " +
                        "Ignore conversion and leave probeset " +
                        "IDs as genes' names?\n\n" +
                        "Answering YES will load the gene " +
                        "expression matrix with probeset IDs, " +
                        "while answering NO will cancel the operation.",
                        "Ignore conversion?",
                        JOptionPane.YES_NO_OPTION,
                        JOptionPane.QUESTION_MESSAGE,
                        null, null,
                        JOptionPane.YES_OPTION);
                if (option == JOptionPane.NO_OPTION) {
                    return;
                }
            }
        }

        genesNames = this.constructGenesNames(
                genesNamesColumn, rowsWithNames);
        conditionsNames = this.constructConditionsNames(conditionsNamesRow,
                columnsWithNames);
        geneExpressionMatrix = this.constructGeneExpressionMatrix(rowsWithNames,
                columnsWithNames);

        NodeInfo expressionMatrixNodeInfo = null;
        NodeInfo preprocessedNodeInfo = null;
        ExpressionMatrix expressionMatrix = new ExpressionMatrix();

        expressionMatrix = new ExpressionMatrix(genesNames,
                                                conditionsNames,
                                                geneExpressionMatrix,
                                                PanelLoading.
                                                DEFAULT_MISSING_VALUE,
                                                organism,
                                                geneID,
                                                dataType);
        TreePath expressionMatrixPath = new TreePath(mainFrame.getTree().
                getRoot());
        try {
            ArrayList<String> infoStrings = new ArrayList<String>();
            infoStrings.add("Organism: ");
            infoStrings.add(organism + "\n");
            infoStrings.add("Datatype: ");
            infoStrings.add(dataType + "\n\n");
            infoStrings.add("Number of genes: ");
            infoStrings.add("" + expressionMatrix.getNumberOfGenes() + "\n");
            infoStrings.add("Number of time points: ");
            infoStrings.add("" + expressionMatrix.
                            getNumberOfConditions() + "\n\n");
            infoStrings.add("Missing values: ");
            infoStrings.add("" + expressionMatrix.hasMissingValues() + "\n");
            infoStrings.add("Missings marked with value: ");
            infoStrings.add("" + expressionMatrix.getMissingValue() + "\n");

            ArrayList<String> infoStyles = new ArrayList<String>();
            for (int i = 0; i < infoStrings.size(); i++) {
                if (i % 2 == 0) {
                    infoStyles.add("bold");
                } else {
                    infoStyles.add("regular");
                }
            }

            expressionMatrixNodeInfo = new NodeInfo(expressionMatrix,
                    NodeInfo.ORIGINAL,
                    mainFrame.getTree().
                    getSelectionPath(),
                    mainFrame.getTree(),
                    null,
                    infoStrings,
                    infoStyles);
            boolean logTransform = logTransformationCheckBox.isSelected();
            expressionMatrixPath = mainFrame.getTree().addLoadedDataset(
                    expressionMatrixNodeInfo);
            logTransformationCheckBox.setSelected(logTransform);
        } catch (InvalidNodeObjectTypeException ex) {
            ex.printStackTrace();
            JOptionPane.showMessageDialog(this,
                                          ex.getMessage(),
                                          "Error loading new dataset",
                                          JOptionPane.ERROR_MESSAGE);
            this.clear();
            return;
        } catch (Exception ex) {
            ex.printStackTrace();
            JOptionPane.showMessageDialog(this,
                                          ex.getMessage(),
                                          "Error loading new dataset: " +
                                          "GO files have been corrupted",
                                          JOptionPane.ERROR_MESSAGE);
            this.clear();
            return;
        }

        if (logTransformationCheckBox.isSelected()) {
            PreProcessedExpressionMatrix preprocessedExpressionMatrix = new
                    PreProcessedExpressionMatrix(expressionMatrix);

            try {
                preprocessedExpressionMatrix.logTransform();
                ArrayList<String> infoStrings = new ArrayList<String>();
                infoStrings.add("Number of genes: ");
                infoStrings.add("" + preprocessedExpressionMatrix.
                                getNumberOfGenes() + "\n");
                infoStrings.add("Number of time points: ");
                infoStrings.add("" + preprocessedExpressionMatrix.
                                getNumberOfConditions() + "\n");
                infoStrings.add("Missing values: ");
                infoStrings.add("" + preprocessedExpressionMatrix.
                                hasMissingValues() + "\n");
                infoStrings.add("Missings marked with value: ");
                infoStrings.add("" + preprocessedExpressionMatrix.
                                getMissingValue() + "\n");
                infoStrings.add("Preprocessing\n");
                infoStrings.add("\tLog transformation: ");
                infoStrings.add("yes");

                ArrayList<String> infoStyles = new ArrayList<String>();
                int i;
                for (i = 0; i < infoStrings.size() - 2; i++) {
                    if (i % 2 == 0) {
                        infoStyles.add("bold");
                    } else {
                        infoStyles.add("regular");
                    }
                }
                infoStyles.add("bold");
                infoStyles.add("regular");
                
                preprocessedNodeInfo = new NodeInfo(
                        preprocessedExpressionMatrix,
                        NodeInfo.PREPROCESSED,
                        expressionMatrixPath,
                        mainFrame.getTree(),
                        expressionMatrixNodeInfo,
                        infoStrings,
                        infoStyles);
                mainFrame.getTree().addNewNode(preprocessedNodeInfo,
                                               expressionMatrixPath, true, true);
            } catch (InvalidNodeObjectTypeException ex) {
                JOptionPane.showMessageDialog(
                        this,
                        ex.getMessage(),
                        "Error creating log transformed expression matrix",
                        JOptionPane.ERROR_MESSAGE);
                this.clear();
                return;
            } catch (Exception ex) {
                ex.printStackTrace();
                JOptionPane.showMessageDialog(this, ex.getMessage(),
                                              "Error computing log transform",
                                              JOptionPane.ERROR_MESSAGE);
                this.clear();
                return;
            }
        } else {
            mainFrame.getTree().setSelectionPath(expressionMatrixPath);
        }
        /**
         * Dataset was successfully loaded.
         */
    }

    public void browseDatasetFileButton_actionPerformed(ActionEvent e) {
        JFileChooser datasetFileChooser = new JFileChooser("Datasets");
        try {
            datasetFileChooser.setFileFilter(new FileFilterExtension(
                    ".txt"));
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        datasetFileChooser.setAcceptAllFileFilterUsed(false);
        int option = datasetFileChooser.showOpenDialog(this);

        if (option == JFileChooser.APPROVE_OPTION) {
            datasetFileTextField.setText(datasetFileChooser.
                                         getSelectedFile().getPath());
        }
    }
}


class PanelLoading_datasetFileTextField_inputMethodAdapter implements
        InputMethodListener {
    private PanelLoading adaptee;
    PanelLoading_datasetFileTextField_inputMethodAdapter(PanelLoading adaptee) {
        this.adaptee = adaptee;
    }

    public void inputMethodTextChanged(InputMethodEvent event) {
        adaptee.datasetFileTextField_inputMethodTextChanged(event);
    }

    public void caretPositionChanged(InputMethodEvent event) {
        adaptee.datasetFileTextField_caretPositionChanged(event);
    }
}


class PanelLoading_datasetFileTextField_caretAdapter implements CaretListener {
    private PanelLoading adaptee;
    PanelLoading_datasetFileTextField_caretAdapter(PanelLoading adaptee) {
        this.adaptee = adaptee;
    }

    public void caretUpdate(CaretEvent e) {
        adaptee.datasetFileTextField_caretUpdate(e);
    }
}


class PanelLoading_geneIDCheckBox_itemAdapter implements ItemListener {
    private PanelLoading adaptee;
    PanelLoading_geneIDCheckBox_itemAdapter(PanelLoading adaptee) {
        this.adaptee = adaptee;
    }

    public void itemStateChanged(ItemEvent e) {
        adaptee.geneIDCheckBox_itemStateChanged(e);
    }
}


class PanelLoading_loadButton_actionAdapter implements ActionListener {
    private PanelLoading adaptee;
    PanelLoading_loadButton_actionAdapter(PanelLoading adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.loadButton_actionPerformed(e);
    }
}


class PanelLoading_browseConversionFileButton_actionAdapter implements
        ActionListener {
    private PanelLoading adaptee;
    PanelLoading_browseConversionFileButton_actionAdapter(PanelLoading adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.browseConversionFileButton_actionPerformed(e);
    }
}

class PanelLoading_browseDatasetFileButton_actionAdapter implements
        ActionListener {
    private PanelLoading adaptee;
    PanelLoading_browseDatasetFileButton_actionAdapter(PanelLoading adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.browseDatasetFileButton_actionPerformed(e);
    }
}
