package biggests.gui;

import javax.swing.*;

import biggests.utils.*;
import de.schlichtherle.io.*;

/**
 * <p>Title: Download gene ontology files thread</p>
 *
 * <p>Description: Thread to download gene ontology files from
 *                 www.geneontology.org (long time consuming task)
 *                 while displaying a progress bar dialog to the user.
 *                 This task always preceeds function analysis, so when
 *                 download of gene ontology files finishes, this thread
 *                 also calls a method to apply biclustering and function
 *                 analysis algorithms.</p>
 *
 * <p>Copyright:   Copyright (C) 2007  Joana Gonçalves
 *                 This program is free software; you can redistribute
 *                 it and/or modify it under the terms of the GNU General
 *                 Public License as published by the Free Software
 *                 Foundation; either version 3 of the License, or
 *                 any later version.
 *
 *                 This program is distributed in the hope that it will
 *                 be useful, but WITHOUT ANY WARRANTY; without even the
 *                 implied warranty of MERCHANTABILITY or FITNESS FOR A
 *                 PARTICULAR PURPOSE.  See the
 *                 <a href="http://www.gnu.org/licenses/gpl.html">
 *                 GNU General Public License</a> for more details.
 * </p>
 *
 * @author Joana P. Gonçalves
 */
public class ThreadDownloadGOFiles extends Thread {
    /**
     * The application's main frame.
     */
    private BiggestsFrame mainFrame;

    /**
     * The function analysis panel who requested the download of the file.
     */
    private PanelFunctionAnalysis functionAnalysisPanel;

    /**
     * The name of the file to download.
     */
    private String filename;

    /**
     * The progress bar dialog.
     */
    private ProgressBarFrame progressFrame;

    /**
     * Creates a new thread to download a gene ontology/association file.
     *
     * @param frame <code>BiggestsFrame</code> the application's main
     *        frame, which contains the URLs of gene ontology/association
     *        files
     * @param panel <code>PanelFunctionAnalysis</code> the function
     *        analysis panel for which the file will be downloaded
     * @param filename <code>String</code> the name of the file to download
     */
    public ThreadDownloadGOFiles(BiggestsFrame frame,
                                 PanelFunctionAnalysis panel, String filename) {
        mainFrame = frame;
        functionAnalysisPanel = panel;
        this.filename = filename;
        progressFrame =
                new ProgressBarFrame(mainFrame, this,
                                     "Connecting to www.geneontology.org", false);
        this.start();
    }

    /**
     * Overrides <code>Thread</code>'s  <code>run</code> method. Performs the
     * actual task of this working thread (download a gene
     * ontology/association file).
     *
     * @see Thread#run()
     */
    public void run() {
        mainFrame.setEnabled(false);
        int result = -5;
        boolean noProblem = false;
        File file = new File(mainFrame.getPathToGO_Files() + File.separator +
                             filename);
        if (file.exists()) {
            noProblem = true;
        }
        try {
            if (filename.equals(PanelFunctionAnalysis.OBO_FILE_NAME)) {
                BiggestsIO.downloadGeneOntologyFile(
                        mainFrame.getGeneOntologyFilesURL(),
                        mainFrame.getPathToGO_Files(),
                        filename,
                        progressFrame);
            } else {
                BiggestsIO.downloadGeneAssociationFile(
                        mainFrame.getGeneAssociationFilesURL(),
                        mainFrame.getPathToGO_Files(),
                        filename,
                        progressFrame);
            }
        } catch (Exception e) {
            if (noProblem) {
                JOptionPane.showMessageDialog(mainFrame,
                                              "File was not updated:\n"
                                              + e.getMessage() + "\n"
                                              +
                                              "Old file remains in the system.",
                                              "File not updated",
                                              JOptionPane.INFORMATION_MESSAGE);
            } else {
                JOptionPane.showMessageDialog(mainFrame,
                                              "File was not downloaded:"
                                              + e.getMessage() + "\n",
                                              "File not downloaded",
                                              JOptionPane.INFORMATION_MESSAGE);
            }
            progressFrame.dispose();
            mainFrame.setEnabled(true);
            mainFrame.setVisible(true);
            return;
        }

        if (filename.equals(PanelFunctionAnalysis.OBO_FILE_NAME)) {
            functionAnalysisPanel.
                    getGeneOntologyFilePathTextField().
                    setText(file.getAbsolutePath());
        } else {
            functionAnalysisPanel.
                    getGeneAssociationFilePathTextField().
                    setText(file.getAbsolutePath());
        }

        progressFrame.dispose();
        mainFrame.setEnabled(true);
        mainFrame.setVisible(true);
    }
}
